/*
   PS2 - Triangle Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to display a triangle to the screen
   using the Official Playstation 2 Linux kit and the sps2 library.
*/


// Demo: PS2 - Triangle.


#include<stdio.h>
#include<sps2lib.h>
#include<sps2tags.h>
#include<sps2util.h>

// Function protoypes...
void RenderScene();


// Structures defining the data we are sending.
typedef struct Vertex { int x, y; };
struct Color { int r, g, b; };


// Triangle vertex data and vertex colors.
#define MAX_POINTS 3

Vertex Triangle[MAX_POINTS] =
{
   { -200, 200 }, { 200, 200 }, { 0, -200 }
};

Color TriColor[MAX_POINTS] =
{
   { 0, 255, 0 }, { 255, 255, 0 }, { 0, 0, 255 }
};


int main()
{
   int SpS2Desc = 0;
   int frames = 0;

   // Initialize sps2 kernel, etc.  Returns a device descriptor.
   SpS2Desc = sps2Init();

   // Error checking.
   if(SpS2Desc < 0)
      {
         printf("sps2 initialize faled...Error!\n");
         exit(0);
      }

   // This will auto detect the hardware for the screen as well as
   // set the appropiate resolution.
   sps2UScreenInit(0);

   // Since we don't have a way to get input we will just
   // render the scene a set number of times instead of rendering
   // until someone does something to exit.  If we force quit
   // the app then the resources will not be released.
   for(frames = 0; frames < 30000; frames++)
      {
          RenderScene();
      }

   // Release all resources.
   sps2UScreenShutdown();

   // Release sps2.
   sps2Release(SpS2Desc);

   return 1;
}


void RenderScene()
{
   int i = 0;

   // GIF = Graphics Synthesiser Interface.  A gif tag tells the
   // graphics synthesiser what primitive to expect.  Right after
   // a gif tag is the actual data.  The gs can draw to the screen a
   // point, line, line strip, triangle, triangle strip, triangle fan,
   // and sprite.

   //
   sps2GIFTag_t GifTag;
   sps2GIFPackedRegister_t GifReg;

   // Clear the scene to a color passed in (in this
   // case 0, 0, 0 means black).
   sps2UScreenClear(0, 0, 0);

   // Initialize the gif tag.
   // In the documentation there is a union with i128 and the rest
   // of the giftag members.  This allows us to set the whole
   // structure to 0 just by setting i128 to 0.
   // NLOOP = basically the number of vertex points we are drawing.
   // PRE = wheter or not to transfer the PRIM register to the GS prim reg.
   // EOP = end of packet.  When it sees 1 here that means this is
   // the last gif packet the GS will see.
   // PRIM = specifies what type of primitve to be drawn.  Triangle = 11.
   // FLG = describes what type of data we have.  Here we are using packed data.
   // NREG = the current register being set.
   GifTag.i128 = 0;
   GifTag.s.NLOOP = 3;
   GifTag.s.PRE = 1;
   GifTag.s.EOP = 1;
   GifTag.s.PRIM = 11;
   GifTag.s.FLG = GIF_FLG_PACKED;
   GifTag.s.NREG = 0;

   // Tell the gif what we are drawing.  We are sending a color then a
   // vertex position.  You must do it in this order since setting the vertex
   // reg XYZ2 will cause a vertex kick.  This means that the vertex in placed
   // on a queue when this happens and what ever color is already set will be
   // used for thet vertex.  If you specify the color after the vertex has been
   // placed on the queue then it will not have the color you wanted. We must
   // add one to the nreg because we are setting a different register with
   // each call.
   SPS2_SET_GIF_REG(GifTag, GifTag.s.NREG++, GIF_REG_RGBAQ);
   SPS2_SET_GIF_REG(GifTag, GifTag.s.NREG++, GIF_REG_XYZ2);

   // Send the gif tag to the hardware.
   DPUT_EE_GIF_FIFO(GifTag.i128);

   // Loop through each point and set its data to the gif tag.
   // Keep in mind that the center of a PS2 screen is 2048, 2048
   // not 0, 0.  Since X, Y data is 12:4 fixed format, we left
   // shift by 4.
   for(i = 0; i < MAX_POINTS; i++)
      {
         // First clear the whole struct by setting i128 to 0.
         // Then set the object color.
         GifReg.i128 = 0;
         GifReg.RGBAQ.s.R = TriColor[i].r;
         GifReg.RGBAQ.s.G = TriColor[i].g;
         GifReg.RGBAQ.s.B = TriColor[i].b;
         GifReg.RGBAQ.s.A = 0x80;

         // Set the register.
         DPUT_EE_GIF_FIFO(GifReg.i128);

         // Do the same with the vertex data.
         GifReg.i128 = 0;
         GifReg.XYZ2.s.X = (Triangle[i].x + 2048) << 4;
         GifReg.XYZ2.s.Y = (Triangle[i].y + 2048) << 4;
         GifReg.XYZ2.s.Z = 1;

         // Set this register.
         DPUT_EE_GIF_FIFO(GifReg.i128);
      }

   // Swap buffers (in other words display to the screen).
   sps2UScreenSwap();
}


// Recap:

// It takes a small amount of work to set up the registers to
// draw a triangle but PS2 was never known for its easy
// development.  I suggest reading the manuals that come
// with the kit and really learn the PS2 in and out.  Its
// a beautiful thing to see a triangle on the T.V. screen
// being powered by the PS2.  Just keep in mind that
// you specify the XYZ2 register last.  The queue is a first
// in first out so when we get to the XYZ2, we need to have
// all the data needed for that vertex point in front of it.


// Copyright February 2005
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com